import { world, system, ItemStack } from "@minecraft/server";

// const collisionBoxes = {
//     "minecraft:zombie": 2 // example distance
// };
world.afterEvents.entitySpawn.subscribe(({ entity }) => {
    const typeFamilyComp = entity.getComponent("minecraft:type_family");
    if (typeFamilyComp && typeFamilyComp.hasTypeFamily("horse")) initializeHorse(entity);
});
world.afterEvents.entityLoad.subscribe(({ entity }) => {
    const typeFamilyComp = entity.getComponent("minecraft:type_family");
    if (typeFamilyComp && typeFamilyComp.hasTypeFamily("horse")) {
        initializeHorse(entity);
    }
    if (typeFamilyComp && typeFamilyComp.hasTypeFamily("cart") && entity.getDynamicProperty("horseID")){
        startCartMovement(entity,entity.getDynamicProperty("horseID"))
    }
});
// world.beforeEvents.entityRemove.subscribe(({ removedEntity }) => {
//     if (removedEntity?.getComponent("minecraft:type_family").hasTypeFamily("cart")) removedEntity.clearDynamicProperties();
// });
function initializeHorse(entity) {
    const typeFamily = entity?.getComponent("minecraft:type_family");
    if (!typeFamily?.hasTypeFamily("horse")) return;

    if (entity.getDynamicProperty("horseID")) return;
    if (entity.getDynamicProperty("hasCart")) return;
    const id = entity.id;
    entity.setDynamicProperty("horseID", id);
    entity.setDynamicProperty("hasCart", false);
    // entity.runCommandAsync(`/say Initialized Horse With ID: ${entity.getDynamicProperty("horseID")}`);
    // entity.runCommandAsync(`/say This Horse Has Cart: ${entity.getDynamicProperty("hasCart")}`);
}

world.beforeEvents.playerInteractWithEntity.subscribe(event => {
    const { player, target, itemStack } = event;

    if (!player.isSneaking) return;
    if (itemStack?.typeId !== "bv_e:cart_wrench") return;

    const typeFamily = target.getComponent("minecraft:type_family");
    const horseID = target.id;
    const isTamed = target.getComponent("minecraft:is_tamed");
    if (typeFamily?.hasTypeFamily("horse") && isTamed) {
        // Check if horse already has a cart
        if (target.getDynamicProperty("hasCart") === true) {
            target.runCommandAsync(`/say §cRemoved Cart From This Horse!!!`);
            target.runCommandAsync("/playsound note.bass @a[r=16] ~~~");
            target.runCommandAsync(`/tag @s remove "${target.getDynamicProperty("horseID")}"`);
            target.setDynamicProperty("hasCart",false);
            target.runCommandAsync(`/tag @e[tag="${target.getDynamicProperty("horseID")}_c"] remove hasHorse`);
            target.runCommandAsync(`/event entity @e[tag="${target.getDynamicProperty("horseID")}_c"] removeCart`);
            target.runCommandAsync(`/tag @e[tag="${target.getDynamicProperty("horseID")}_c"] remove "${target.getDynamicProperty("horseID")}_c"`);
            target.runCommandAsync(`/event entity @s removeCart`);
            player.clearDynamicProperties();
            const overworld = world.getDimension("overworld");
            let entities = overworld.getEntities();
            for (const entity of entities) {
                const typeFamilyComp = entity.getComponent("minecraft:type_family");
                if (typeFamilyComp && typeFamilyComp.hasTypeFamily("cart") && entity.getDynamicProperty("horseID") == target.getDynamicProperty("horseID")){
                    // entity.runCommandAsync(`/say The Cart With ID ${entity.getDynamicProperty("horseID")} Has Been Clear`);
                    entity.clearDynamicProperties();
                    startCartMovement(entity, null);
                }
            }
            return;
        }

        const storedID = target.getDynamicProperty("horseID");
        if (!storedID) return;

        // Bind horse to cart
        player.setDynamicProperty("horseID", storedID);
        target.runCommandAsync(`tag @s add "${storedID}"`);
        target.runCommandAsync(`/say §aSelected Horse`);
        target.runCommandAsync("/playsound random.click @a[r=16] ~~~");
        target.setDynamicProperty("hasCart",true);
        return;
    }

    // If target is a cart and player has no cartID, set the cartID
    if (typeFamily?.hasTypeFamily("cart") && player.getDynamicProperty("horseID")) {
        target.setDynamicProperty("horseID", player.getDynamicProperty("horseID"));
        startCartMovement(target, player.getDynamicProperty("horseID"));
        target.runCommandAsync(`/say §aAttached Cart`);
        target.runCommandAsync("/playsound random.levelup @a[r=16] ~~~");
        target.runCommandAsync(`/tag @s add hasHorse`);
        target.runCommandAsync(`/event entity @s hasCart`);
        target.runCommandAsync(`/event entity @e[family=horse,tag="${target.getDynamicProperty("horseID")}"] hasCart`);
        target.runCommandAsync(`/tag @s add "${target.getDynamicProperty("horseID")}_c"`);
        player.clearDynamicProperties();
        return;
    } else {
        target.runCommandAsync("/playsound note.bass @a[r=16] ~~~");
        target.runCommandAsync(`/say §cYou Need To Select A Horse First!!!`)
    }
});


function startCartMovement(cart, id) {
    // Stop any old interval
    if (cart.__intervalId) {
        system.clearRun(cart.__intervalId);
    }

    if (!id) {
        cart.runCommandAsync(`/say §7Cart movement cleared (no horse ID)`);
        return;
    }

    cart.__intervalId = system.runInterval(() => {
        const health = cart.getComponent("minecraft:health");
        if (!health || health.currentValue <= 0) {
            cart.runCommandAsync(`/say §cCart dead or invalid`);
            system.clearRun(cart.__intervalId);
            cart.__intervalId = undefined;
            return;
        }

        cart.runCommandAsync(`execute if entity @e[tag="${id}", rm=3.2] run tp @s[tag=hasHorse] ^^^0.5 facing @e[tag="${id}", rm=3.2]`);
        cart.runCommandAsync(`execute unless block ~~0.5~ tall_grass run execute unless block ~~1~ air run tp @s[tag=hasHorse] ^^1^ facing @e[tag="${id}", r=5]`);
        cart.runCommandAsync(`tp @s[tag=hasHorse] ^^-1^ facing @e[tag="${id}", r=5] true`);
    }); // every 0.25 seconds
}


//thieeus getEntities để clear dynamic property (CArt)