import { world, system } from "@minecraft/server";

// Track carts and their remaining time in ticks (10 seconds = 200 ticks)
const cartsToCheck = new Map();

world.afterEvents.playerInteractWithEntity.subscribe(event => {
    const { target } = event;

    // Check if it's a cart
    const typeFamily = target.getComponent("minecraft:type_family");
    if (!typeFamily?.hasTypeFamily("cart")) return;

    const inventoryComp = target.getComponent("minecraft:inventory");
    if (!inventoryComp) return;

    // Add or refresh the countdown to 200 ticks
    cartsToCheck.set(target, 200);
});

// Run every tick (you can make this 20 for once-per-second if needed)
system.runInterval(() => {
    for (const [cart, timeLeft] of cartsToCheck.entries()) {
        if (!cart?.isValid()) {
            cartsToCheck.delete(cart);
            continue;
        }

        // Decrement time
        const newTime = timeLeft - 1;
        if (newTime <= 0) {
            cartsToCheck.delete(cart); // Stop tracking
            continue;
        } else {
            cartsToCheck.set(cart, newTime); // Update remaining time
        }

        // Inventory check
        const inventoryComp = cart.getComponent("minecraft:inventory");
        if (!inventoryComp) continue;

        const container = inventoryComp.container;
        const totalSlots = container.size;
        let emptySlots = 0;

        for (let i = 0; i < totalSlots; i++) {
            const item = container.getItem(i);
            if (!item) emptySlots++;
        }

        const third = totalSlots / 3;
        const thirdsEmpty = Math.floor(emptySlots / third);

        // Trigger event
        switch (thirdsEmpty) {
            case 3:
                cart.runCommandAsync("/event entity @s empty");
                break;
            case 2:
                cart.runCommandAsync("/event entity @s abitfull");
                break;
            case 1:
                cart.runCommandAsync("/event entity @s halffull");
                break;
            default:
                cart.runCommandAsync("/event entity @s full");
                break;
        }
    }
}, 1); // Check every tick (or change to 20 for every second)
