import { world, system, ItemStack} from "@minecraft/server"
import { ActionFormData, ActionFormResponse, ModalFormData } from '@minecraft/server-ui';
import 'cart'
import 'cart_inventory'

world.beforeEvents.playerInteractWithEntity.subscribe(event => {
    const { player, target, itemStack } = event;
    const typeFamily = target.getComponent("minecraft:type_family");
    // Check if the target entity is of type 'bv_e:horse'
    if (typeFamily && typeFamily.hasTypeFamily("horse")) {
        const rideable = target.getComponent("rideable");
        if (!rideable) return;
        const moveModeProperty = target.getProperty("move_mode:property");
        if (moveModeProperty !== "interact") return;

        const riders = rideable.getRiders();
        const hasPlayerRider = riders.some(rider => rider.typeId === "minecraft:player");

        if (hasPlayerRider) {
            system.run(() => moveModeSelect(player,target))
        }
    }
});

function moveModeSelect(player,target){
    const form = new ActionFormData()
        .title("§l§nGaits Mode Select")
        .body(`§oSelect Between Gaits Mode to apply the moving mode and speed when riding a horse.`)
        .button("Walk")
        .button("Trot")
        .button("Canter")
        .button("Gallop")
        .button("Close");

    form.show(player).then(r => {
        if (r.canceled) {
            return;
        };
        if (r.selection === 0) {
            player.runCommand("/event entity @e[r=3,c=1,family=horse] walk")
        };
        if (r.selection === 1) {
            player.runCommand("/event entity @e[r=3,c=1,family=horse] trot")
            if (Math.random() < 0.5){
                player.runCommand("/playanimation @e[r=3,family=horse,c=1] animation.horse_model.stand")
                player.runCommand("/effect @e[r=3,family=horse,c=1] slowness 2 255 true")
            }
        };
        if (r.selection === 2) {
            player.runCommand("/event entity @e[r=3,c=1,family=horse] canter")
                if (Math.random() < 0.5){
                    player.runCommand("/playanimation @e[r=3,family=horse,c=1] animation.horse_model.stand")
                    player.runCommand("/effect @e[r=3,family=horse,c=1] slowness 2 255 true")
                }
        };
        if (r.selection === 3) {
            if (!target.getDynamicProperty("hasCart")){
                player.runCommand("/event entity @e[r=3,c=1,family=horse] gallop")
                if (Math.random() < 0.5){
                    player.runCommand("/playanimation @e[r=3,family=horse,c=1] animation.horse_model.stand")
                    player.runCommand("/effect @e[r=3,family=horse,c=1] slowness 2 255 true")
                }
            } else {
                player.runCommand("/say Cannot Gallop While Pulling A Cart")
            }  
        };
    });
}

world.beforeEvents.itemUse.subscribe(data => {
    let player = data.source
    let title = "§lHorse Settings"
    if (data.itemStack.typeId === "bv_e:golden_horseshoes"){
        system.run(() => main(player))
    }

    function main(){
        const form = new ActionFormData()
        .title(title)
        .body(`§r§oYou Can Change Some Aspects Of The Addon Via This Settings Tab\n\nRemember: Stand near a horse of your choice to apply the changed settings.`)
        .button(`Camera Settings`, "textures/icons/camera")
        // .button(`Horse Gaits Control`, "textures/icons/horse_gait")
        .button(`Close`)
        form.show(player).then(r => {
            if (r.canceled) {
                return;
            };
            if (r.selection === 0) {
                cameraSetting(player);
            }
            // if (r.selection === 1) {
            //     gaitSetting(player);
            // }
        })

        function cameraSetting(player){
            const form = new ActionFormData()
                .title("§l§nCamera Controll")
                .body(`§oSelect Between FreeCam Mode And Normal Mode to apply the camera mode when riding a horse.`)
                .button("Free Cam Mode")
                .button("Normal Mode")
                .button("Camera FIX")
                .button("Back")
                .button("Close");

            form.show(player).then(r => {
                if (r.canceled) {
                    return;
                };
                if (r.selection === 0) {
                    player.runCommand("/event entity @e[r=3,c=1,family=horse] free_cam")
                    player.runCommand("/tag @s add free_cam")
                };
                if (r.selection === 1) {
                    player.runCommand("/event entity @e[r=3,c=1,family=horse] normal")
                    player.runCommand("/tag @s remove free_cam")
                };
                if (r.selection === 2) {
                    // player.runCommand("/camera @s clear")
                    player.runCommand("/tag @s remove free_cam")
                };
                if (r.selection === 3) {
                    main(player)
                };
            });
        }

        function gaitSetting(player){
            const form = new ActionFormData()
                .title("§l§nHorse Gaits Controll")
                .body(`§oSelect Between Interact Mode And Jump Button Mode to apply the gaits controll mode when riding a horse.`)
                .button("Interact Mode")
                .button("Jump Button Mode")
                .button("Back")
                .button("Close");

            form.show(player).then(r => {
                if (r.canceled) {
                    return;
                };
                if (r.selection === 0) {
                    player.runCommand("/event entity @e[r=3,c=1,family=horse] interact")
                };
                if (r.selection === 1) {
                    player.runCommand("/event entity @e[r=3,c=1,family=horse] jump")
                };
                if (r.selection === 2) {
                    main(player)
                };
            });
        }
    }
});